@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2005-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 2005-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
@@ -21,14 +21,8 @@
 **
 ****************************************************************************/
 
-/****************************************************************
-**
-** Definition of CannonField class, Qt tutorial 9
-**
-****************************************************************/
-
-#ifndef CANNON_H
-#define CANNON_H
+#ifndef CANNONFIELD_H
+#define CANNONFIELD_H
 
 #include <QWidget>
 
@@ -54,4 +48,4 @@ private:
     int currentAngle;
 };
 
-#endif // CANNON_H
+#endif
