@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -72,7 +72,7 @@ QThreadData *QThreadData::get(QThread *thread)
 
 QThreadPrivate::QThreadPrivate()
     : QObjectPrivate(), running(false), finished(false), terminated(false),
-      stackSize(0)
+      stackSize(0), priority(QThread::InheritPriority)
 {
 #if defined (Q_OS_UNIX)
     thread_id = 0;
@@ -438,6 +438,35 @@ void QThread::cleanup()
     Use isRunning() instead.
 */
 
+/*! \fn void QThread::setPriority(Priority priority)
+    \since 4.1
+
+    This function sets the \a priority for a running thread. If the
+    thread is not running, this function does nothing and returns
+    immediately.  Use start() to start a thread with a specific
+    priority.
+
+    The \a priority argument can be any value in the \c
+    QThread::Priority enum except for \c InheritPriorty.
+
+    \sa Priority priority() start()
+*/
+
+/*!
+    \since 4.1
+
+    Returns the priority for a running thread.  If the thread is not
+    running, this function returns \c InheritPriority.
+
+    \sa Priority setPriority() start()
+*/
+QThread::Priority QThread::priority() const
+{
+    Q_D(const QThread);
+    QMutexLocker locker(&d->mutex);
+    return d->priority;
+}
+
 #else // QT_NO_THREAD
 
 QThread* QThread::instance = 0;
