@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -120,7 +120,7 @@ public:
     void setLabelText(DialogLabel label, const QString &text);
     QString labelText(DialogLabel label) const;
 
-signals:
+Q_SIGNALS:
     void filesSelected(const QStringList &files);
     void currentChanged(const QString &path);
 
@@ -233,6 +233,7 @@ private:
     Q_PRIVATE_SLOT(d_func(), void sortBySize())
     Q_PRIVATE_SLOT(d_func(), void sortByDate())
     Q_PRIVATE_SLOT(d_func(), void setUnsorted())
+    Q_PRIVATE_SLOT(d_func(), void sortByColumn(int))
     Q_PRIVATE_SLOT(d_func(), void currentChanged(const QModelIndex &index))
 };
 
@@ -244,4 +245,3 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(QFileDialog::Options)
 #endif // QT_NO_FILEDIALOG
 
 #endif // QFILEDIALOG_H
-
