@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -108,10 +108,11 @@ QLayout::QLayout(QLayoutPrivate &dd, QLayout *lay, QWidget *w)
         lay->addItem(this);
     } else if (w) {
         if (w->layout()) {
-            qWarning("QLayout \"%s\" added to %s \"%s\", which already has a"
-                     " layout. This may cause memory leaks.",
+            qWarning("Attempting to add QLayout \"%s\" to %s \"%s\", which already has a"
+                     " layout.",
                      qPrintable(QObject::objectName()), w->metaObject()->className(),
                      w->objectName().toLocal8Bit().data());
+            setParent(0);
         } else {
             d->topLevel = true;
             w->d_func()->layout = this;
@@ -939,7 +940,7 @@ void QLayout::update()
 bool QLayout::activate()
 {
     Q_D(QLayout);
-    if (!parent())
+    if (!d->enabled || !parent())
         return false;
     if (!d->topLevel)
         return static_cast<QLayout*>(parent())->activate();
@@ -948,7 +949,7 @@ bool QLayout::activate()
     QWidget *mw = static_cast<QWidget*>(parent());
     if (mw == 0) {
         qWarning("QLayout::activate: %s \"%s\" does not have a main widget",
-                  QObject::metaObject()->className(), QObject::objectName().toLocal8Bit().data());
+                 QObject::metaObject()->className(), QObject::objectName().toLocal8Bit().data());
         return false;
     }
     activateRecursiveHelper(this);
@@ -971,10 +972,10 @@ bool QLayout::activate()
         mw->setMinimumSize(totalMinimumSize());
         mw->setMaximumSize(totalMaximumSize());
         break;
-    case SetDefaultConstraint:
+    case SetDefaultConstraint: {
+        bool widthSet = explMin & Qt::Horizontal;
+        bool heightSet = explMin & Qt::Vertical;
         if (mw->isWindow()) {
-            bool widthSet = explMin & Qt::Horizontal;
-            bool heightSet = explMin & Qt::Vertical;
             QSize ms = totalMinimumSize();
             if (widthSet)
                 ms.setWidth(mw->minimumSize().width());
@@ -990,8 +991,16 @@ bool QLayout::activate()
                 }
             }
             mw->setMinimumSize(ms);
+        } else if (!widthSet || !heightSet) {
+            QSize ms = mw->minimumSize();
+            if (!widthSet)
+                ms.setWidth(0);
+            if (!heightSet)
+                ms.setHeight(0);
+            mw->setMinimumSize(ms);
         }
         break;
+    }
     case SetNoConstraint:
         break;
     }
