@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -268,6 +268,9 @@ static void heuristicSetGlyphAttributes(QShaperItem *item, const QChar *uc, int
             glyphs[pos].attributes = glyphs[pos-1].attributes;
             ++pos;
         }
+        // hide soft-hyphens by default
+        if (uc[i].unicode() == 0x00ad)
+            glyphs[pos].attributes.dontPrint = true;
         const QUnicodeTables::Properties *prop = QUnicodeTables::properties(uc[i].unicode());
         int cat = prop->category;
         if (cat != QChar::Mark_NonSpacing) {
