@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -52,21 +52,8 @@ public:
 
 static QWidget *qt_sizegrip_topLevelWidget(QWidget* w)
 {
-    QWidget *p = w->parentWidget();
-    while (!w->isWindow() && p && !p->inherits("QWorkspace")) {
-        w = p;
-        p = p->parentWidget();
-    }
-    return w;
-}
-
-static QWidget* qt_sizegrip_workspace(QWidget* w)
-{
-    while (w && !w->inherits("QWorkspace")) {
-        if (w->isWindow())
-            return 0;
+    while (!w->isWindow() && !(w->parentWidget()->windowType() == Qt::SubWindow))
         w = w->parentWidget();
-    }
     return w;
 }
 
@@ -94,8 +81,7 @@ static QWidget* qt_sizegrip_workspace(QWidget* w)
     On some platforms the sizegrip automatically hides itself when the
     window is shown full screen or maximised.
 
-    \inlineimage qsizegrip-m.png Screenshot in Motif style
-    \inlineimage qsizegrip-w.png Screenshot in Windows style
+    \image plastique-sizegrip.png A size grip widget shown in the Plastique widget style.
 
     \sa QStatusBar QWidget::windowState()
 */
@@ -139,7 +125,7 @@ void QSizeGripPrivate::init()
 #endif
     q->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed));
 #if defined(Q_WS_X11)
-    if (!qt_sizegrip_workspace(q)) {
+    if (qt_sizegrip_topLevelWidget(q)->isWindow()) {
         WId id = q->winId();
         XChangeProperty(X11->display, q->window()->winId(),
                         ATOM(_QT_SIZEGRIP), XA_WINDOW, 32, PropModeReplace,
@@ -218,8 +204,8 @@ void QSizeGrip::mouseMoveEvent(QMouseEvent * e)
 
     QPoint np(e->globalPos());
 
-    QWidget* ws = qt_sizegrip_workspace(this);
-    if (ws) {
+    if (!tlw->isWindow()) {
+        QWidget* ws = tlw->parentWidget()->parentWidget();
         QPoint tmp(ws->mapFromGlobal(np));
         if (tmp.x() > ws->width())
             tmp.setX(ws->width());
@@ -299,7 +285,7 @@ bool QSizeGrip::event(QEvent *e)
     switch(e->type()) {
     case QEvent::Hide:
     case QEvent::Show:
-        if(!QApplication::closingDown() && parentWidget() && !qt_sizegrip_workspace(this)) {
+        if(!QApplication::closingDown() && parentWidget()) {
             if(QWidget *w = qt_sizegrip_topLevelWidget(this)) {
                 if(w->isWindow())
                     QWidgetPrivate::qt_mac_update_sizer(w, e->type() == QEvent::Hide ? -1 : 1);
