@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -38,6 +38,7 @@
 #include "private/q3membuf_p.h"
 #include "qevent.h"
 #include "q3url.h"
+#include "qhttp.h"
 
 //#define Q3HTTP_DEBUG
 
@@ -1188,7 +1189,7 @@ void Q3Http::init()
 {
     bytesRead = 0;
     d = new Q3HttpPrivate;
-    d->errorString = tr( "Unknown error" );
+    d->errorString = QHttp::tr( "Unknown error" );
 
     connect( &d->socket, SIGNAL( connected() ),
 	    this, SLOT( slotConnected() ) );
@@ -1385,7 +1386,7 @@ void Q3Http::abort()
     if ( r == 0 )
 	return;
 
-    finishedWithError( tr("Request aborted"), Aborted );
+    finishedWithError( QHttp::tr("Request aborted"), Aborted );
     clearPendingRequests();
     d->socket.clearPendingData();
     close();
@@ -1419,7 +1420,7 @@ Q_LONG Q3Http::readBlock( char *data, Q_ULONG maxlen )
 #endif
 	return -1;
     }
-    if ( maxlen >= d->rba.size() )
+    if ( maxlen >= (Q_ULONG)d->rba.size() )
 	maxlen = d->rba.size();
     d->rba.consumeBytes( maxlen, data );
 
@@ -1755,7 +1756,7 @@ void Q3Http::startNextRequest()
 	return;
 
     d->error = NoError;
-    d->errorString = tr( "Unknown error" );
+    d->errorString = QHttp::tr( "Unknown error" );
 
     if ( bytesAvailable() )
 	readAll(); // clear the data
@@ -1766,7 +1767,7 @@ void Q3Http::startNextRequest()
 void Q3Http::sendRequest()
 {
     if ( d->hostname.isNull() ) {
-	finishedWithError( tr("No server set to connect to"), UnknownError );
+	finishedWithError( QHttp::tr("No server set to connect to"), UnknownError );
 	return;
     }
 
@@ -1822,11 +1823,11 @@ void Q3Http::slotClosed()
 	if ( d->response.hasKey( "content-length" ) ) {
 	    // We got Content-Length, so did we get all bytes?
 	    if ( d->bytesDone+bytesAvailable() != d->response.contentLength() ) {
-		finishedWithError( tr("Wrong content length"), WrongContentLength );
+		finishedWithError( QHttp::tr("Wrong content length"), WrongContentLength );
 	    }
 	}
     } else if ( d->state == Connecting || d->state == Sending ) {
-	finishedWithError( tr("Server closed connection unexpectedly"), UnexpectedClose );
+	finishedWithError( QHttp::tr("Server closed connection unexpectedly"), UnexpectedClose );
     }
 
     d->postDevice = 0;
@@ -1864,13 +1865,13 @@ void Q3Http::slotError( int err )
     if ( d->state == Connecting || d->state == Reading || d->state == Sending ) {
 	switch ( err ) {
 	    case Q3Socket::ErrConnectionRefused:
-		finishedWithError( tr("Connection refused"), ConnectionRefused );
+		finishedWithError( QHttp::tr("Connection refused"), ConnectionRefused );
 		break;
 	    case Q3Socket::ErrHostNotFound:
-		finishedWithError( tr("Host %1 not found").arg(d->socket.peerName()), HostNotFound );
+		finishedWithError( QHttp::tr("Host %1 not found").arg(d->socket.peerName()), HostNotFound );
 		break;
 	    default:
-		finishedWithError( tr("HTTP request failed"), UnknownError );
+		finishedWithError( QHttp::tr("HTTP request failed"), UnknownError );
 		break;
 	}
     }
@@ -1939,7 +1940,7 @@ void Q3Http::slotReadyRead()
 #endif
 	// Check header
 	if ( !d->response.isValid() ) {
-	    finishedWithError( tr("Invalid HTTP response header"), InvalidResponseHeader );
+	    finishedWithError( QHttp::tr("Invalid HTTP response header"), InvalidResponseHeader );
 	    close();
 	    return;
 	}
@@ -1979,7 +1980,7 @@ void Q3Http::slotReadyRead()
 			bool ok;
 			d->chunkedSize = sizeString.toInt( &ok, 16 );
 			if ( !ok ) {
-			    finishedWithError( tr("Invalid HTTP chunked body"), WrongContentLength );
+			    finishedWithError( QHttp::tr("Invalid HTTP chunked body"), WrongContentLength );
 			    close();
 			    return;
 			}
@@ -2025,7 +2026,7 @@ void Q3Http::slotReadyRead()
 			char tmp[2];
 			d->socket.readBlock( tmp, 2 );
 			if ( tmp[0] != '\r' || tmp[1] != '\n' ) {
-			    finishedWithError( tr("Invalid HTTP chunked body"), WrongContentLength );
+			    finishedWithError( QHttp::tr("Invalid HTTP chunked body"), WrongContentLength );
 			    close();
 			    return;
 			}
@@ -2318,13 +2319,13 @@ void Q3Http::clientStateChanged( int state )
     if ( url() ) {
 	switch ( (State)state ) {
 	    case Connecting:
-		emit connectionStateChanged( ConHostFound, tr( "Host %1 found" ).arg( url()->host() ) );
+		emit connectionStateChanged( ConHostFound, QHttp::tr( "Host %1 found" ).arg( url()->host() ) );
 		break;
 	    case Sending:
-		emit connectionStateChanged( ConConnected, tr( "Connected to host %1" ).arg( url()->host() ) );
+		emit connectionStateChanged( ConConnected, QHttp::tr( "Connected to host %1" ).arg( url()->host() ) );
 		break;
 	    case Unconnected:
-		emit connectionStateChanged( ConClosed, tr( "Connection to %1 closed" ).arg( url()->host() ) );
+		emit connectionStateChanged( ConClosed, QHttp::tr( "Connection to %1 closed" ).arg( url()->host() ) );
 		break;
 	    default:
 		break;
@@ -2332,13 +2333,13 @@ void Q3Http::clientStateChanged( int state )
     } else {
 	switch ( (State)state ) {
 	    case Connecting:
-		emit connectionStateChanged( ConHostFound, tr( "Host found" ) );
+		emit connectionStateChanged( ConHostFound, QHttp::tr( "Host found" ) );
 		break;
 	    case Sending:
-		emit connectionStateChanged( ConConnected, tr( "Connected to host" ) );
+		emit connectionStateChanged( ConConnected, QHttp::tr( "Connected to host" ) );
 		break;
 	    case Unconnected:
-		emit connectionStateChanged( ConClosed, tr( "Connection closed" ) );
+		emit connectionStateChanged( ConClosed, QHttp::tr( "Connection closed" ) );
 		break;
 	    default:
 		break;
