@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -24,10 +24,10 @@
 #ifndef Q3TEXTEDIT_H
 #define Q3TEXTEDIT_H
 
-#include "Qt3Support/q3scrollview.h"
-#include "Qt3Support/q3stylesheet.h"
-#include "Qt3Support/q3mimefactory.h"
-#include "QtCore/qmap.h"
+#include <Qt3Support/q3scrollview.h>
+#include <Qt3Support/q3stylesheet.h>
+#include <Qt3Support/q3mimefactory.h>
+#include <QtCore/qmap.h>
 
 QT_MODULE(Qt3SupportLight)
 
@@ -290,7 +290,7 @@ public:
     AutoFormatting autoFormatting() const;
     Q3SyntaxHighlighter *syntaxHighlighter() const;
 
-public slots:
+public Q_SLOTS:
 #ifndef QT_NO_MIME
     virtual void setMimeSourceFactory(Q3MimeSourceFactory* factory);
 #endif
@@ -382,7 +382,7 @@ public slots:
     int maxLogLines() const;
 #endif
 
-signals:
+Q_SIGNALS:
     void textChanged();
     void selectionChanged();
     void copyAvailable(bool);
@@ -431,11 +431,11 @@ protected:
     virtual Q3PopupMenu *createPopupMenu();
     void drawCursor(bool visible);
 
-protected slots:
+protected Q_SLOTS:
     virtual void doChangeInterval();
     virtual void sliderReleased();
 
-private slots:
+private Q_SLOTS:
     void formatMore();
     void doResize();
     void autoScrollTimerDone();
@@ -447,7 +447,7 @@ private slots:
 
 private:
     struct Q_COMPAT_EXPORT UndoRedoInfo {
-        enum Type { Invalid, Insert, Delete, Backspace, Return, RemoveSelected, Format, Style };
+        enum Type { Invalid, Insert, Delete, Backspace, Return, RemoveSelected, Format, Style, IME };
 
         UndoRedoInfo(Q3TextDocument *dc);
         ~UndoRedoInfo();
@@ -525,7 +525,7 @@ private:
     void optimCheckLimit(const QString& str);
     bool optimHasBoldMetrics(int line);
 
-private slots:
+private Q_SLOTS:
     void optimDoAutoScroll();
 #endif // QT_TEXTEDIT_OPTIMIZATION
 
