@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -31,6 +31,7 @@
 #include "qpainter.h"
 #include "qmap.h"
 #include "q3mainwindow.h"
+#include "q3toolbar.h"
 
 //#define QDOCKAREA_DEBUG
 
@@ -400,6 +401,8 @@ int Q3DockAreaLayout::layoutItems(const QRect &rect, bool testonly)
             continue;
         hadResizable = hadResizable || dw->isResizeEnabled();
         dw->updateSplitterVisibility(visibleWindows > 1); //!dw->area()->isLastDockWindow(dw));
+        if (Q3ToolBar *tb = qobject_cast<Q3ToolBar *>(dw))
+            tb->checkForExtension(dw->size());
     }
     return sectionpos + linestrut;
 }
