@@ -915,7 +915,7 @@ static void parseFont(QSvgNode *node,
             }
         }
         if (!fontStyle)
-            fontStyle = new QSvgFontStyle(font);
+            fontStyle = new QSvgFontStyle(font, node->document());
         if (!anchor.isEmpty())
             fontStyle->setTextAnchor(anchor);
 
@@ -2345,7 +2345,7 @@ static QSvgNode *createSvgNode(QSvgNode *parent,
     QString heightStr = attributes.value("height");
     QString viewBoxStr = attributes.value("viewBox");
 
-    QSvgHandler::LengthType type;
+    QSvgHandler::LengthType type = QSvgHandler::PX; // FIXME: is the default correct?
     qreal width = 0;
     if (!widthStr.isEmpty()) {
         width = parseLength(widthStr, type, handler);
@@ -2593,8 +2593,8 @@ bool QSvgHandler::startElement(const QString &namespaceURI,
             QString id = attributes.value("id");
             if (id.isEmpty())
                 id = attributes.value("xml:id");
-            m_nodes.top()->appendStyleProperty(prop, id, true);
             m_style = prop;
+            m_nodes.top()->appendStyleProperty(prop, id, true);
         } else {
             qWarning("Couldn't parse node: %s", qPrintable(localName));
         }
