@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
@@ -23,8 +23,8 @@
 
 #include "ui3reader.h"
 
-#include <QtXml/QDomElement>
-#include <QtCore/QFile>
+#include <QDomElement>
+#include <QFile>
 
 void Ui3Reader::computeDeps(const QDomElement &e,
         QStringList &globalIncludes,
@@ -108,4 +108,3 @@ void Ui3Reader::computeDeps(const QDomElement &e,
         }
     }
 }
-
