@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -26,15 +26,15 @@
 
 #include <QtCore/qglobal.h>
 
-#define QT_EXTENSION_EXTERN Q_DECL_EXPORT
-#define QT_EXTENSION_IMPORT Q_DECL_IMPORT
+#define QDESIGNER_EXTENSION_EXTERN Q_DECL_EXPORT
+#define QDESIGNER_EXTENSION_IMPORT Q_DECL_IMPORT
 
 #ifdef QT_DESIGNER_STATIC
-#  define QT_EXTENSION_EXPORT
-#elif defined(QT_EXTENSION_LIBRARY)
-#  define QT_EXTENSION_EXPORT QT_EXTENSION_EXTERN
+#  define QDESIGNER_EXTENSION_EXPORT
+#elif defined(QDESIGNER_EXTENSION_LIBRARY)
+#  define QDESIGNER_EXTENSION_EXPORT QDESIGNER_EXTENSION_EXTERN
 #else
-#  define QT_EXTENSION_EXPORT QT_EXTENSION_IMPORT
+#  define QDESIGNER_EXTENSION_EXPORT QDESIGNER_EXTENSION_IMPORT
 #endif
 
 #endif // EXTENSION_GLOBAL_H
