@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the Qt Linguist of the Qt Toolkit.
 **
@@ -42,23 +42,29 @@ extern void fetchtr_ui( const char *fileName, MetaTranslator *tor,
 
 // defined in merge.cpp
 extern void merge( MetaTranslator *tor, const MetaTranslator *virginTor,
-                   bool verbose );
+                   bool verbose, bool noObsolete );
 
 typedef QList<MetaTranslatorMessage> TML;
 
+static const char *g_defaultExtensions = "ui,c,c++,cc,cpp,cxx,ch,h,h++,hh,hpp,hxx";
+
 static void printUsage()
 {
     fprintf( stderr, "Usage:\n"
-             "    lupdate [options] project-file\n"
-             "    lupdate [options] source-files -ts ts-files\n"
+        "    lupdate [options] [project-file]\n"
+             "    lupdate [options] [source-file|path]... -ts ts-files\n"
              "Options:\n"
-             "    -help  Display this information and exit\n"
+             "    -help  Display this information and exit.\n"
              "    -noobsolete\n"
-             "           Drop all obsolete strings\n"
+             "           Drop all obsolete strings.\n"
+             "    -extensions <ext>[,<ext>]...\n"
+             "           Process files with the given extensions only.\n"
+             "           The extension list must be separated with commas, not with whitespace.\n"
+             "           Default: '%s'.\n"
              "    -verbose\n"
-             "           Explain what is being done\n"
+             "           Explain what is being done.\n"
              "    -version\n"
-             "           Display the version of lupdate and exit\n" );
+             "           Display the version of lupdate and exit.\n", g_defaultExtensions);
 }
 
 static void updateTsFiles( const MetaTranslator& fetchedTor,
@@ -73,7 +79,7 @@ static void updateTsFiles( const MetaTranslator& fetchedTor,
             tor.setCodec( codecForTr.toLatin1() );
         if ( verbose )
             fprintf( stderr, "Updating '%s'...\n", (*t).toLatin1().data() );
-        merge( &tor, &fetchedTor, verbose );
+        merge( &tor, &fetchedTor, verbose, noObsolete );
         if ( noObsolete )
             tor.stripObsoleteMessages();
         tor.stripEmptyContexts();
@@ -92,6 +98,24 @@ static void updateTsFiles( const MetaTranslator& fetchedTor,
     }
 }
 
+void recursiveFileInfoList(const QDir &dir, const QStringList &nameFilters, QDir::Filters filter, bool recursive, QFileInfoList *fileinfolist)
+{
+    if (recursive) filter |= QDir::AllDirs;
+    QFileInfoList entries = dir.entryInfoList(nameFilters, filter);
+
+    QFileInfoList::iterator it;
+    for (it = entries.begin(); it != entries.end(); ++it) {
+        QString fname = it->fileName();
+        if (fname != QLatin1String(".") && fname != QLatin1String("..")) {
+            if (it->isDir()) {
+                recursiveFileInfoList(QDir(it->absoluteFilePath()), nameFilters, filter, recursive, fileinfolist);
+            }else {
+                fileinfolist->append(*it);
+            }
+        }
+    }
+}
+
 int main( int argc, char **argv )
 {
     QString defaultContext = "@default";
@@ -108,6 +132,8 @@ int main( int argc, char **argv )
     bool metTsFlag = false;
 
     int i;
+    QString extensions = QLatin1String(g_defaultExtensions);
+    QStringList extensionsNameFilters;
 
     for ( i = 1; i < argc; i++ ) {
         if ( qstrcmp(argv[i], "-ts") == 0 )
@@ -130,13 +156,21 @@ int main( int argc, char **argv )
         } else if ( qstrcmp(argv[i], "-ts") == 0 ) {
             metTsFlag = true;
             continue;
+        } else if ( qstrcmp(argv[i], "-extensions") == 0 ) {
+            ++i;
+            if (i == argc) {
+                fprintf( stderr, "The -extensions option should be followed by an extension list.");
+                return 1;
+            }
+            extensions = QString(QLatin1String(argv[i]));
+            continue;
         }
 
         numFiles++;
 
         QString fullText;
 
-        if ( !metTsFlag ) {
+        if ( standardSyntax && !metTsFlag ) {
             QFile f( argv[i] );
             if ( !f.open(QIODevice::ReadOnly) ) {
 #if defined(_MSC_VER) && _MSC_VER >= 1400
@@ -157,55 +191,57 @@ int main( int argc, char **argv )
         }
 
         QString oldDir = QDir::currentPath();
-        QDir::setCurrent( QFileInfo(argv[i]).path() );
 
         if ( standardSyntax ) {
+            QDir::setCurrent( QFileInfo(argv[i]).path() );
             fetchedTor = MetaTranslator();
             codecForTr.clear();
 			codecForSource.clear();
             tsFileNames.clear();
 
-            QMap<QString, QString> tagMap = proFileTagMap( fullText );
-            QMap<QString, QString>::Iterator it;
+            QMap<QString, QString> tagMap;
+            if (proFileTagMap( fullText, &tagMap )) {
+                QMap<QString, QString>::Iterator it;
 
-            for ( it = tagMap.begin(); it != tagMap.end(); ++it ) {
-                QStringList toks = tokenizeFileNames(it.value());
-                QStringList::Iterator t;
+                for ( it = tagMap.begin(); it != tagMap.end(); ++it ) {
+                    QStringList toks = tokenizeFileNames(it.value());
+                    QStringList::Iterator t;
 
-                for ( t = toks.begin(); t != toks.end(); ++t ) {
-                    if ( it.key() == "HEADERS" || it.key() == "SOURCES" ) {
-                        fetchtr_cpp( (*t).toAscii(), &fetchedTor, defaultContext.toAscii(), true, codecForSource );
-                        metSomething = true;
-                    } else if ( it.key() == "INTERFACES" ||
-                                it.key() == "FORMS" ) {
-                        fetchtr_ui( (*t).toAscii(), &fetchedTor, defaultContext.toAscii(), true );
-                        fetchtr_cpp( (*t).toAscii() + ".h", &fetchedTor, defaultContext.toAscii(), false, codecForSource );
-                        metSomething = true;
-                    } else if ( it.key() == "TRANSLATIONS" ) {
-                        tsFileNames.append( *t );
-                        metSomething = true;
-                    } else if ( it.key() == "CODEC" ||
-                                it.key() == "DEFAULTCODEC" ||
-                                it.key() == "CODECFORTR" ) {
-                        codecForTr = (*t).toLatin1();
-                    } else if ( it.key() == "CODECFORSRC" ) {
-                        codecForSource = (*t).toLatin1();
+                    for ( t = toks.begin(); t != toks.end(); ++t ) {
+                        if ( it.key() == "HEADERS" || it.key() == "SOURCES" ) {
+                            fetchtr_cpp( (*t).toAscii(), &fetchedTor, defaultContext.toAscii(), true, codecForSource );
+                            metSomething = true;
+                        } else if ( it.key() == "INTERFACES" ||
+                                    it.key() == "FORMS" ) {
+                            fetchtr_ui( (*t).toAscii(), &fetchedTor, defaultContext.toAscii(), true );
+                            fetchtr_cpp( (*t).toAscii() + ".h", &fetchedTor, defaultContext.toAscii(), false, codecForSource );
+                            metSomething = true;
+                        } else if ( it.key() == "TRANSLATIONS" ) {
+                            tsFileNames.append( *t );
+                            metSomething = true;
+                        } else if ( it.key() == "CODEC" ||
+                                    it.key() == "DEFAULTCODEC" ||
+                                    it.key() == "CODECFORTR" ) {
+                            codecForTr = (*t).toLatin1();
+                        } else if ( it.key() == "CODECFORSRC" ) {
+                            codecForSource = (*t).toLatin1();
+                        }
                     }
                 }
-            }
 
-            updateTsFiles( fetchedTor, tsFileNames, codecForTr, noObsolete, verbose );
+                updateTsFiles( fetchedTor, tsFileNames, codecForTr, noObsolete, verbose );
 
-            if ( !metSomething ) {
-                fprintf( stderr,
-                         "lupdate warning: File '%s' does not look like a"
-                         " project file\n",
-                         argv[i] );
-            } else if ( tsFileNames.isEmpty() ) {
-                fprintf( stderr,
-                         "lupdate warning: Met no 'TRANSLATIONS' entry in"
-                         " project file '%s'\n",
-                         argv[i] );
+                if ( !metSomething ) {
+                    fprintf( stderr,
+                             "lupdate warning: File '%s' does not look like a"
+                             " project file\n",
+                             argv[i] );
+                } else if ( tsFileNames.isEmpty() ) {
+                    fprintf( stderr,
+                             "lupdate warning: Met no 'TRANSLATIONS' entry in"
+                             " project file '%s'\n",
+                             argv[i] );
+                }
             }
         } else {
             if ( metTsFlag ) {
@@ -226,12 +262,44 @@ int main( int argc, char **argv )
                 }
             } else {
                 QFileInfo fi(argv[i]);
-                if ( QString(argv[i]).toLower().endsWith(".ui") ) {
-                    fetchtr_ui( fi.fileName().toAscii(), &fetchedTor, defaultContext.toAscii(), true );
-                    fetchtr_cpp( fi.fileName().toAscii() + ".h", &fetchedTor,
-                                 defaultContext.toAscii(), false, codecForSource );
-                } else {
-                    fetchtr_cpp( fi.fileName().toAscii(), &fetchedTor, defaultContext.toAscii(), true, codecForSource );
+                if (fi.isDir()) {
+                    if ( verbose ) fprintf(stderr, "Scanning directory '%s'...\n", argv[i]);
+                    QDir dir = QDir(fi.filePath());
+                    if (extensionsNameFilters.isEmpty()) {
+                        extensions = extensions.trimmed();
+                        // Remove the potential dot in front of each extension
+                        if (extensions.startsWith('.'))
+                            extensions.remove(0,1);
+                        extensions.replace(",.", ",");
+
+                        extensions.insert(0, QLatin1String("*."));
+                        extensions.replace(',', QLatin1String(",*."));
+                        extensionsNameFilters = extensions.split(',');
+                    }
+                    QDir::Filters filters = QDir::Files | QDir::NoSymLinks;
+                    QFileInfoList fileinfolist;
+                    recursiveFileInfoList(dir, extensionsNameFilters, filters, true, &fileinfolist);
+                    QFileInfoList::iterator ii;
+                    QByteArray fn;
+                    for (ii = fileinfolist.begin(); ii != fileinfolist.end(); ++ii) {
+                        fn = ii->filePath().toLatin1();
+#ifdef LINGUIST_DEBUG
+                        fprintf(stderr, "%s\n", fn.data());
+#endif
+                        if (fn.toLower().endsWith(".ui") ) {
+                            fetchtr_ui( fn, &fetchedTor, defaultContext.toAscii(), true );
+                        } else {
+                            fetchtr_cpp( fn, &fetchedTor, defaultContext.toAscii(), true, codecForSource );
+                        }
+                    }
+                }else{
+                    if ( QString(argv[i]).toLower().endsWith(".ui") ) {
+                        fetchtr_ui( fi.filePath().toAscii(), &fetchedTor, defaultContext.toAscii(), true );
+                        fetchtr_cpp( fi.filePath().toAscii() + ".h", &fetchedTor,
+                                     defaultContext.toAscii(), false, codecForSource );
+                    } else {
+                        fetchtr_cpp( fi.filePath().toAscii(), &fetchedTor, defaultContext.toAscii(), true, codecForSource );
+                    }
                 }
             }
         }
