@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
@@ -30,13 +30,14 @@ class QImage;
 class QTimer;
 class QAnimationWriter;
 struct QVFbHeader;
+class QVFbViewProtocol;
 
 class QVFbView : public QWidget
 {
     Q_OBJECT
 public:
     enum Rotation { Rot0, Rot90, Rot180, Rot270 };
-    QVFbView( int display_id, int w, int h, int d, Rotation r, QWidget *parent = 0);
+    QVFbView( int id, int w, int h, int d, Rotation r, QWidget *parent = 0);
     ~QVFbView();
 
     int displayId() const;
@@ -64,17 +65,17 @@ public:
     double zoomV() const { return vzm; }
 
     QSize sizeHint() const;
+    void setRate(int);
 
 public slots:
     void setTouchscreenEmulation( bool );
     void setLcdScreenEmulation( bool );
-    void setRate( int );
     void setZoom( double, double );
     void startAnimation( const QString& );
     void stopAnimation();
 
 protected slots:
-    void flushChanges();
+    void refreshDisplay(const QRect &);
 
 protected:
     QImage getBuffer( const QRect &r, int &leading ) const;
@@ -95,9 +96,6 @@ protected:
 
 private:
     void setDirty( const QRect& );
-    int shmId;
-    unsigned char *data;
-    QVFbHeader *hdr;
     int viewdepth; // "faked" depth
     int rsh;
     int gsh;
@@ -109,17 +107,11 @@ private:
     int contentsHeight;
     double gred, ggreen, gblue;
     QRgb* gammatable;
-    int lockId;
-    QTimer *t_flush;
 
-    int mouseFd;
-    int keyboardFd;
     int refreshRate;
-    QString mousePipe;
-    QString keyboardPipe;
     QAnimationWriter *animation;
-    int displayid;
     double hzm,vzm;
+    QVFbViewProtocol *mView;
     bool emulateTouchscreen;
     bool emulateLcdScreen;
     Rotation rotation;
